==> file_storage_handler_abm-5.0.62598a0.tar.gz: file_storage_handler_abm-5.0.62598a0/file_storage_handler/memory_zipper.py <==
#  Copyright (C) 2024
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
import asyncio
import io
import logging
import typing
import zipfile
from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass
from secrets import token_hex

from file_storage.exceptions import PathDoesNotExists

from .database import Database
from .extra import AnySession, assert_raise
from .link_keeper import LinkKeeper
from .entities.link_info import LinkInfo

logger = logging.getLogger(__name__)
LI = typing.TypeVar('LI', bound=LinkInfo)


class MaxZipFilesError(Exception):
    pass


class MaxZipSizeError(Exception):
    pass


class MemoryZipper(typing.Generic[LI]):
    @dataclass
    class Config:
        executors_amount: int = 2
        max_zip_files: int = 100
        max_zip_size: int = 100 * 1024 * 1024

    @dataclass
    class Context:
        link_keeper: LinkKeeper
        database: Database

    def __init__(self, context: Context, config: Config = None) -> None:
        self.config = config or self.Config()
        self.context = context
        self._pool_executor = ThreadPoolExecutor(max_workers=self.config.executors_amount)

    async def zip_files(self, link_ids: list[str], session: AnySession = None) -> tuple[LI, bytes]:
        logger.info(f'Zip links: {link_ids=}')
        assert_raise(len(link_ids) <= self.config.max_zip_files,
                     MaxZipFilesError(f"Max files amount limit exceeded: {len(link_ids)}/{self.config.max_zip_files}"))

        async with self.context.database.ensure_session(session) as session:
            link_infos = await self.context.link_keeper.head_multiple(link_ids, session)
            assert_raise(not (missing_link_ids := {i.id for i in link_infos} - set(link_ids)),
                         PathDoesNotExists(f'Some files not found: {missing_link_ids}'))
            assert_raise((total_size := sum(i.file_info.size for i in link_infos)) < self.config.max_zip_size,
                         MaxZipSizeError(f"Max files size limit exceeded: {total_size}/{self.config.max_zip_size}"))

        zip_buffer = io.BytesIO()

        with zipfile.ZipFile(zip_buffer, 'w', zipfile.ZIP_DEFLATED, False) as zip_file:
            for info in link_infos:
                # If file name contains '/' zipfile lib thinking that it's subdirectories,
                # splitting name by it and creating archive with subdirectories
                # e.g. 'foo/bar.pdf' - in archive will be created folder foo and inside it file bar.
                file_payload = await self.context.link_keeper.read(info.file_info_key)
                await asyncio.get_event_loop().run_in_executor(
                    self._pool_executor, zip_file.writestr, info.filename.replace('/', '-'), file_payload)

        zip_buffer.seek(0)
        file_content = zip_buffer.read()
        zip_extension = 'zip'
        filename = f'{token_hex(16)}.{zip_extension}'
        info = LinkInfo(
            id=None,
            file_info_key='',
            filename=filename,
            extension=zip_extension
        )

        return info, file_content

==> file_storage_handler_abm-8.0.72395a0.tar.gz: file_storage_handler_abm-8.0.72395a0/file_storage_handler/memory_zipper.py <==
#  Copyright (C) 2024
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors:
#  Mike Orlov <m.orlov@abm-jsc.ru>
#  Andrey Vaydich <a.vaydich@abm-jsc.ru>

import asyncio
import io
import logging
import typing
import zipfile
from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass
from secrets import token_hex

from file_storage.exceptions import PathDoesNotExists

from .database import Database
from .extra import AnySession, assert_raise
from .link_keeper import LinkKeeper
from .entities.link_info import LinkInfo
from .tools.convert_file_name import convert_file_name

logger = logging.getLogger(__name__)
LI = typing.TypeVar('LI', bound=LinkInfo)


class MaxZipFilesError(Exception):
    pass


class MaxZipSizeError(Exception):
    pass


class MemoryZipper(typing.Generic[LI]):
    @dataclass
    class Config:
        executors_amount: int = 2
        max_zip_files: int = 100
        max_zip_size: int = 100 * 1024 * 1024
        max_file_name_bytes_size: int = 150

    @dataclass
    class Context:
        link_keeper: LinkKeeper
        database: Database

    def __init__(self, context: Context, config: Config = None) -> None:
        self.config = config or self.Config()
        self.context = context
        self._pool_executor = ThreadPoolExecutor(max_workers=self.config.executors_amount)

    async def zip_files(self, link_ids: list[str], session: AnySession = None) -> tuple[LI, bytes]:
        logger.info(f'Zip links: {link_ids=}')
        assert_raise(len(link_ids) <= self.config.max_zip_files,
                     MaxZipFilesError(f"Max files amount limit exceeded: {len(link_ids)}/{self.config.max_zip_files}"))

        async with self.context.database.ensure_session(session) as session:
            link_infos = await self.context.link_keeper.head_multiple(link_ids, session)
            assert_raise(not (missing_link_ids := {i.id for i in link_infos} - set(link_ids)),
                         PathDoesNotExists(f'Some files not found: {missing_link_ids}'))
            assert_raise((total_size := sum(i.file_info.size for i in link_infos)) < self.config.max_zip_size,
                         MaxZipSizeError(f"Max files size limit exceeded: {total_size}/{self.config.max_zip_size}"))

        zip_buffer = io.BytesIO()

        with zipfile.ZipFile(zip_buffer, 'w', zipfile.ZIP_DEFLATED, False) as zip_file:
            for info in link_infos:
                file_payload = await self.context.link_keeper.read(info.file_info_key)
                await asyncio.get_event_loop().run_in_executor(
                    self._pool_executor,
                    zip_file.writestr,
                    convert_file_name(info.filename, self.config.max_file_name_bytes_size),
                    file_payload
                )

        zip_buffer.seek(0)
        file_content = zip_buffer.read()
        zip_extension = 'zip'
        filename = f'{token_hex(16)}.{zip_extension}'
        info = LinkInfo(
            id=None,
            file_info_key='',
            filename=filename,
            extension=zip_extension
        )

        return info, file_content
